@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: connect.c,v 1.46 2003/01/09 14:52:51 bagder Exp $
+ * $Id: connect.c,v 1.56 2003/05/13 12:12:17 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -81,8 +81,7 @@
 #include "memdebug.h"
 #endif
 
-static
-int geterrno(void)
+int Curl_ourerrno(void)
 {
 #ifdef WIN32
   return (int)GetLastError();
@@ -144,7 +143,11 @@ int Curl_nonblock(int socket,    /* operate on this */
 }
 
 /*
- * Return 0 on fine connect, -1 on error and 1 on timeout.
+ * waitconnect() returns:
+ * 0    fine connect
+ * -1   select() error
+ * 1    select() timeout
+ * 2    select() returned with an error condition
  */
 static
 int waitconnect(int sockfd, /* socket */
@@ -194,10 +197,6 @@ static CURLcode bindlocal(struct connectdata *conn,
 
 #ifdef HAVE_INET_NTOA
 
-#ifndef INADDR_NONE
-#define INADDR_NONE (in_addr_t) ~0
-#endif
-
   struct SessionHandle *data = conn->data;
 
   /*************************************************************
@@ -210,7 +209,11 @@ static CURLcode bindlocal(struct connectdata *conn,
     char myhost[256] = "";
     in_addr_t in;
 
-    if(Curl_if2ip(data->set.device, myhost, sizeof(myhost))) {
+    /* First check if the given name is an IP address */
+    in=inet_addr(data->set.device);
+      
+    if((in == CURL_INADDR_NONE) &&
+       Curl_if2ip(data->set.device, myhost, sizeof(myhost))) {
       /*
        * We now have the numerical IPv4-style x.y.z.w in the 'myhost' buffer
        */
@@ -243,22 +246,31 @@ static CURLcode bindlocal(struct connectdata *conn,
     infof(data, "We bind local end to %s\n", myhost);
 
     in=inet_addr(myhost);
-    if (INADDR_NONE != in) {
+    if (CURL_INADDR_NONE != in) {
 
       if ( h ) {
         Curl_addrinfo *addr = h->addr;
 
-        Curl_resolv_unlock(h);
+        Curl_resolv_unlock(data, h);
         /* we don't need it anymore after this function has returned */
 
-        memset((char *)&sa, 0, sizeof(sa));
 #ifdef ENABLE_IPV6
-        memcpy((char *)&sa.sin_addr, addr->ai_addr, addr->ai_addrlen);        
-        sa.sin_family = addr->ai_family;
+        (void)sa; /* prevent compiler warning */
+        if( bind(sockfd, addr->ai_addr, addr->ai_addrlen) >= 0) {
+          /* we succeeded to bind */
+          struct sockaddr_in6 add;
+	
+          size = sizeof(add);
+          if(getsockname(sockfd, (struct sockaddr *) &add,
+                         (socklen_t *)&size)<0) {
+            failf(data, "getsockname() failed");
+            return CURLE_HTTP_PORT_FAILED;
+          }
+        }
 #else
+        memset((char *)&sa, 0, sizeof(sa));
         memcpy((char *)&sa.sin_addr, addr->h_addr, addr->h_length);
         sa.sin_family = AF_INET;
-#endif
         sa.sin_addr.s_addr = in;
         sa.sin_port = 0; /* get any port */
 	
@@ -273,6 +285,7 @@ static CURLcode bindlocal(struct connectdata *conn,
             return CURLE_HTTP_PORT_FAILED;
           }
         }
+#endif
         else {
           switch(errno) {
           case EBADF:
@@ -336,7 +349,7 @@ int socketerror(int sockfd)
 
   if( -1 == getsockopt(sockfd, SOL_SOCKET, SO_ERROR,
                        (void *)&err, &errSize))
-    err = geterrno();
+    err = Curl_ourerrno();
   
   return err;
 }
@@ -396,10 +409,15 @@ CURLcode Curl_is_connected(struct connectdata *conn,
       return CURLE_OK;
     }
     /* nope, not connected for real */
-    if(err)
-      return CURLE_COULDNT_CONNECT;
+    failf(data, "Connection failed, socket error: %d", err);
+    return CURLE_COULDNT_CONNECT;
+  }
+  else if(1 != rc) {
+    int error = Curl_ourerrno();
+    failf(data, "Failed connect to %s:%d, errno: %d",
+          conn->hostname, conn->port, error);
+    return CURLE_COULDNT_CONNECT;
   }
-
   /*
    * If the connection phase is "done" here, we should attempt to connect
    * to the "next address" in the Curl_hostaddr structure that we resolved
@@ -507,7 +525,7 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
       rc = connect(sockfd, ai->ai_addr, ai->ai_addrlen);
 
       if(-1 == rc) {
-        int error=geterrno();
+        int error=Curl_ourerrno();
 
         switch (error) {
         case EINPROGRESS:
@@ -548,6 +566,9 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
         failf(data, "socket error: %d", err);
         /* we are _not_ connected, it was a false alert, continue please */
       }
+      else if(2 == rc)
+        /* waitconnect() returned error */
+        ;
       else if(data->state.used_interface == Curl_if_multi) {
         /* When running the multi interface, we bail out here */
         rc = 0;
@@ -623,7 +644,7 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
                  sizeof(serv_addr));
 
     if(-1 == rc) {
-      int error=geterrno();
+      int error=Curl_ourerrno();
 
       switch (error) {
       case EINPROGRESS:
