@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: ftp.c,v 1.166 2003/01/07 11:25:44 bagder Exp $
+ * $Id: ftp.c,v 1.181 2003/05/14 06:31:00 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -158,6 +158,7 @@ static CURLcode AllowServerConnect(struct SessionHandle *data,
       infof(data, "Connection accepted from server\n");
 
       conn->secondarysocket = s;
+      Curl_nonblock(s, TRUE); /* enable non-blocking */
     }
     break;
   }
@@ -173,7 +174,7 @@ static CURLcode AllowServerConnect(struct SessionHandle *data,
  * response and extract the relevant return code for the invoking function.
  */
 
-CURLcode Curl_GetFTPResponse(int *nreadp, /* return number of bytes read */
+CURLcode Curl_GetFTPResponse(ssize_t *nreadp, /* return number of bytes read */
                              struct connectdata *conn,
                              int *ftpcode) /* return the ftp-code */
 {
@@ -237,7 +238,7 @@ CURLcode Curl_GetFTPResponse(int *nreadp, /* return number of bytes read */
 
     if(!ftp->cache) {
       readfd = rkeepfd;		   /* set every lap */
-      interval.tv_sec = timeout;
+      interval.tv_sec = 1; /* use 1 second timeout intervals */
       interval.tv_usec = 0;
 
       switch (select (sockfd+1, &readfd, NULL, NULL, &interval)) {
@@ -246,9 +247,10 @@ CURLcode Curl_GetFTPResponse(int *nreadp, /* return number of bytes read */
         failf(data, "Transfer aborted due to select() error: %d", errno);
         break;
       case 0: /* timeout */
-        result = CURLE_OPERATION_TIMEDOUT;
-        failf(data, "Transfer aborted due to timeout");
-        break;
+        if(Curl_pgrsUpdate(conn))
+          return CURLE_ABORTED_BY_CALLBACK;
+        continue; /* just continue in our loop for the timeout duration */
+
       default:
         break;
       }
@@ -387,7 +389,7 @@ CURLcode Curl_GetFTPResponse(int *nreadp, /* return number of bytes read */
 CURLcode Curl_ftp_connect(struct connectdata *conn)
 {
   /* this is FTP and no proxy */
-  int nread;
+  ssize_t nread;
   struct SessionHandle *data=conn->data;
   char *buf = data->state.buffer; /* this is our buffer */
   struct FTP *ftp;
@@ -726,7 +728,10 @@ CURLcode ftp_cwd(struct connectdata *conn, char *path)
   if (result)
     return result;
 
-  if (ftpcode != 250) {
+  /* According to RFC959, CWD is supposed to return 250 on success, but
+     there seem to be non-compliant FTP servers out there that return 200,
+     so we accept any '2xy' code here. */
+  if (ftpcode/100 != 2) {
     failf(conn->data, "Couldn't cd to %s", path);
     return CURLE_FTP_ACCESS_DENIED;
   }
@@ -933,6 +938,7 @@ ftp_pasv_verbose(struct connectdata *conn,
 #  endif
         
 # else
+  (void)hostent_buf; /* avoid compiler warning */
   answer = gethostbyaddr((char *) &address, sizeof(address), AF_INET);
 # endif
 #else
@@ -961,7 +967,7 @@ ftp_pasv_verbose(struct connectdata *conn,
 #else
   const int niflags = NI_NUMERICHOST | NI_NUMERICSERV;
 #endif
-  port = 0; /* unused, prevent warning */
+  (void)port; /* prevent compiler warning */
   if (getnameinfo(addr->ai_addr, addr->ai_addrlen,
                   nbuf, sizeof(nbuf), sbuf, sizeof(sbuf), niflags)) {
     snprintf(nbuf, sizeof(nbuf), "?");
@@ -1076,7 +1082,8 @@ CURLcode ftp_use_port(struct connectdata *conn)
     return CURLE_FTP_PORT_FAILED;
   }
 
-  for (modep = (char **)mode; modep && *modep; modep++) {
+  for (modep = (char **)(data->set.ftp_use_eprt?&mode[0]:&mode[2]);
+       modep && *modep; modep++) {
     int lprtaf, eprtaf;
     int alen=0, plen=0;
     
@@ -1206,7 +1213,13 @@ CURLcode ftp_use_port(struct connectdata *conn)
   bool sa_filled_in = FALSE;
 
   if(data->set.ftpport) {
-    if(Curl_if2ip(data->set.ftpport, myhost, sizeof(myhost))) {
+    in_addr_t in;
+
+    /* First check if the given name is an IP address */
+    in=inet_addr(data->set.ftpport);
+
+    if((in == CURL_INADDR_NONE) &&
+       Curl_if2ip(data->set.ftpport, myhost, sizeof(myhost))) {
       h = Curl_resolv(data, myhost, 0);
     }
     else {
@@ -1233,7 +1246,7 @@ CURLcode ftp_use_port(struct connectdata *conn)
 
   if(h)
     /* when we return from here, we can forget about this */
-    Curl_resolv_unlock(h);
+    Curl_resolv_unlock(data, h);
 
   if ( h || sa_filled_in) {
     if( (portsock = socket(AF_INET, SOCK_STREAM, 0)) >= 0 ) {
@@ -1497,7 +1510,7 @@ CURLcode ftp_use_pasv(struct connectdata *conn,
                             &conninfo,
                             connected);
 
-  Curl_resolv_unlock(addr); /* we're done using this address */
+  Curl_resolv_unlock(data, addr); /* we're done using this address */
 
   /*
    * When this is used from the multi interface, this might've returned with
@@ -1854,9 +1867,19 @@ CURLcode Curl_ftp_nextconnect(struct connectdata *conn)
 
       int size=-1; /* default unknown size */
 
+
+      /*
+       * It appears that there are FTP-servers that return size 0 for files
+       * when SIZE is used on the file while being in BINARY mode. To work
+       * around that (stupid) behavior, we attempt to parse the RETR response
+       * even if the SIZE returned size zero.
+       *
+       * Debugging help from Salvatore Sorrentino on February 26, 2003.
+       */
+
       if(!dirlist &&
          !data->set.ftp_ascii &&
-         (-1 == downloadsize)) {
+         (downloadsize < 1)) {
         /*
          * It seems directory listings either don't show the size or very
          * often uses size 0 anyway. ASCII transfers may very well turn out
@@ -1953,17 +1976,46 @@ CURLcode ftp_perform(struct connectdata *conn,
       return result;
   }
 
-  /* change directory first! */
-  if(ftp->dir && ftp->dir[0]) {
-    if ((result = ftp_cwd(conn, ftp->dir)) != CURLE_OK)
+  {
+    int i; /* counter for loop */
+    for (i=0; ftp->dirs[i]; i++) {
+      /* RFC 1738 says empty components should be respected too */
+      if ((result = ftp_cwd(conn, ftp->dirs[i])) != CURLE_OK)
         return result;
+    }
   }
 
-  /* Requested time of file? */
-  if(data->set.get_filetime && ftp->file) {
+  /* Requested time of file or time-depended transfer? */
+  if((data->set.get_filetime || data->set.timecondition) &&
+     ftp->file) {
     result = ftp_getfiletime(conn, ftp->file);
     if(result)
       return result;
+
+    if(data->set.timecondition) {
+      if((data->info.filetime > 0) && (data->set.timevalue > 0)) {
+        switch(data->set.timecondition) {
+        case TIMECOND_IFMODSINCE:
+        default:
+          if(data->info.filetime < data->set.timevalue) {
+            infof(data, "The requested document is not new enough\n");
+            ftp->no_transfer = TRUE; /* mark this to not transfer data */
+            return CURLE_OK;
+          }
+          break;
+        case TIMECOND_IFUNMODSINCE:
+          if(data->info.filetime > data->set.timevalue) {
+            infof(data, "The requested document is not old enough\n");
+            ftp->no_transfer = TRUE; /* mark this to not transfer data */
+            return CURLE_OK;
+          }
+          break;
+        } /* switch */
+      }
+      else {
+        infof(data, "Skipping time comparison\n");
+      }
+    }
   }
 
   /* If we have selected NOBODY and HEADER, it means that we only want file
@@ -2006,7 +2058,7 @@ CURLcode ftp_perform(struct connectdata *conn,
       tm = localtime((unsigned long *)&data->info.filetime);
 #endif
       /* format: "Tue, 15 Nov 1994 12:45:26 GMT" */
-      strftime(buf, BUFSIZE-1, "Last-Modified: %a, %d %b %Y %H:%M:%S %Z\r\n",
+      strftime(buf, BUFSIZE-1, "Last-Modified: %a, %d %b %Y %H:%M:%S GMT\r\n",
                tm);
       result = Curl_client_write(data, CLIENTWRITE_BOTH, buf, 0);
       if(result)
@@ -2051,33 +2103,70 @@ CURLcode ftp_perform(struct connectdata *conn,
  */
 CURLcode Curl_ftp(struct connectdata *conn)
 {
-  CURLcode retcode;
-  bool connected;
+  CURLcode retcode=CURLE_OK;
+  bool connected=0;
 
   struct SessionHandle *data = conn->data;
   struct FTP *ftp;
-  int dirlength=0; /* 0 forces strlen() */
+
+  char *slash_pos;  /* position of the first '/' char in curpos */
+  char *cur_pos=conn->ppath; /* current position in ppath. point at the begin
+                                of next path component */
+  int path_part=0;/* current path component */
 
   /* the ftp struct is already inited in ftp_connect() */
   ftp = conn->proto.ftp;
+  conn->size = -1; /* make sure this is unknown at this point */
 
-  /* We split the path into dir and file parts *before* we URLdecode
-     it */
-  ftp->file = strrchr(conn->ppath, '/');
-  if(ftp->file) {
-    if(ftp->file != conn->ppath)
-      dirlength=ftp->file-conn->ppath; /* don't count the traling slash */
+  Curl_pgrsSetUploadCounter(data, 0);
+  Curl_pgrsSetDownloadCounter(data, 0);
+  Curl_pgrsSetUploadSize(data, 0);
+  Curl_pgrsSetDownloadSize(data, 0);
 
-    ftp->file++; /* point to the first letter in the file name part or
-                    remain NULL */
-  }
-  else {
-    ftp->file = conn->ppath; /* there's only a file part */
+  /*  fixed : initialize ftp->dirs[xxx] to NULL !
+      is done in Curl_ftp_connect() */
+
+  /* parse the URL path into separate path components */
+  while((slash_pos=strchr(cur_pos, '/'))) {
+    /* seek out the next path component */
+    if (0 == slash_pos-cur_pos)  /* empty path component, like "x//y" */
+      ftp->dirs[path_part] = strdup(""); /* empty string */
+    else
+      ftp->dirs[path_part] = curl_unescape(cur_pos,slash_pos-cur_pos);
+    
+    if (!ftp->dirs[path_part]) { /* run out of memory ... */
+      failf(data, "no memory");
+      retcode = CURLE_OUT_OF_MEMORY;
+    }
+    else {
+      cur_pos = slash_pos + 1; /* jump to the rest of the string */
+      if(++path_part >= (CURL_MAX_FTP_DIRDEPTH-1)) {
+        /* too deep, we need the last entry to be kept NULL at all
+           times to signal end of list */
+        failf(data, "too deep dir hierarchy");
+        retcode = CURLE_URL_MALFORMAT;
+      }
+    }
+    if (retcode) {
+      int i;
+      for (i=0;i<path_part;i++) { /* free previous parts */
+        free(ftp->dirs[i]);
+        ftp->dirs[i]=NULL;
+      }
+      return retcode; /* failure */
+    }
   }
 
+  ftp->file = cur_pos;  /* the rest is the file name */
+
   if(*ftp->file) {
     ftp->file = curl_unescape(ftp->file, 0);
     if(NULL == ftp->file) {
+      int i;
+      for (i=0;i<path_part;i++){
+        free(ftp->dirs[i]);
+        ftp->dirs[i]=NULL;
+      }
       failf(data, "no memory");
       return CURLE_OUT_OF_MEMORY;
     }
@@ -2085,28 +2174,22 @@ CURLcode Curl_ftp(struct connectdata *conn)
   else
     ftp->file=NULL; /* instead of point to a zero byte, we make it a NULL
                        pointer */
-
-  ftp->urlpath = conn->ppath;
-  if(dirlength) {
-    ftp->dir = curl_unescape(ftp->urlpath, dirlength);
-    if(NULL == ftp->dir) {
-      if(ftp->file)
-        free(ftp->file);
-      failf(data, "no memory");
-      return CURLE_OUT_OF_MEMORY; /* failure */
-    }
-  }
-  else
-    ftp->dir = NULL;
-
+  
   retcode = ftp_perform(conn, &connected);
 
   if(CURLE_OK == retcode) {
     if(connected)
       retcode = Curl_ftp_nextconnect(conn);
-    else
-      /* since we didn't connect now, we want do_more to get called */
-      conn->bits.do_more = TRUE;
+    else {
+      if(ftp->no_transfer) {
+        /* no data to transfer */
+        retcode=Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);        
+      }
+      else {
+        /* since we didn't connect now, we want do_more to get called */
+        conn->bits.do_more = TRUE;
+      }
+    }
   }
 
   return retcode;
@@ -2171,6 +2254,7 @@ CURLcode Curl_ftpsendf(struct connectdata *conn,
 CURLcode Curl_ftp_disconnect(struct connectdata *conn)
 {
   struct FTP *ftp= conn->proto.ftp;
+  int i;
 
   /* The FTP session may or may not have been allocated/setup at this point! */
   if(ftp) {
@@ -2180,20 +2264,14 @@ CURLcode Curl_ftp_disconnect(struct connectdata *conn)
       free(ftp->cache);
     if(ftp->file)
       free(ftp->file);
-    if(ftp->dir)
-      free(ftp->dir);
+    for (i=0;ftp->dirs[i];i++){
+      free(ftp->dirs[i]);
+      ftp->dirs[i]=NULL;
+    }
 
-    ftp->file = ftp->dir = NULL; /* zero */
+    ftp->file = NULL; /* zero */
   }
   return CURLE_OK;
 }
 
-/*
- * local variables:
- * eval: (load-file "../curl-mode.el")
- * end:
- * vim600: fdm=marker
- * vim: et sw=2 ts=2 sts=2 tw=78
- */
-
 #endif /* CURL_DISABLE_FTP */
