@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: ssluse.c,v 1.59 2003/01/07 16:33:11 bagder Exp $
+ * $Id: ssluse.c,v 1.72 2003/05/01 13:37:36 bagder Exp $
  ***************************************************************************/
 
 /*
@@ -30,10 +30,17 @@
 
 #include <string.h>
 #include <stdlib.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
 
 #include "urldata.h"
 #include "sendf.h"
 #include "formdata.h" /* for the boundary function */
+#include "url.h" /* for the ssl config check function */
 
 #ifdef USE_SSLEAY
 #include <openssl/rand.h>
@@ -141,7 +148,8 @@ int random_the_seed(struct SessionHandle *data)
   {
     /* If there's an option and a define, the option overrides the
        define */
-    int ret = RAND_egd(data->set.ssl.egdsocket?data->set.ssl.egdsocket:EGD_SOCKET);
+    int ret = RAND_egd(data->set.ssl.egdsocket?
+                       data->set.ssl.egdsocket:EGD_SOCKET);
     if(-1 != ret) {
       nread += ret;
       if(seed_enough(nread))
@@ -159,14 +167,24 @@ int random_the_seed(struct SessionHandle *data)
 #else
   {
     int len;
-    char *area = Curl_FormBoundary();
-    if(!area)
-      return 3; /* out of memory */
-	
-    len = strlen(area);
-    RAND_seed(area, len);
+    char *area;
 
-    free(area); /* now remove the random junk */
+    /* Changed call to RAND_seed to use the underlying RAND_add implementation
+     * directly.  Do this in a loop, with the amount of additional entropy
+     * being dependent upon the algorithm used by Curl_FormBoundary(): N bytes
+     * of a 7-bit ascii set. -- Richard Gorton, March 11 2003.
+     */
+	
+    do {
+      area = Curl_FormBoundary();
+      if(!area)
+        return 3; /* out of memory */
+	
+      len = strlen(area);
+      RAND_add(area, len, (len >> 1));
+
+      free(area); /* now remove the random junk */
+    } while (!RAND_status());
   }
 #endif
 
@@ -504,8 +522,9 @@ static int Get_SSL_Session(struct connectdata *conn,
     if(!check->sessionid)
       /* not session ID means blank entry */
       continue;
-    if(strequal(conn->name, check->name) &&
-       (conn->remote_port == check->remote_port) ) {
+    if(curl_strequal(conn->name, check->name) &&
+       (conn->remote_port == check->remote_port) &&
+       Curl_ssl_config_matches(&conn->ssl_config, &check->ssl_config)) {
       /* yes, we have a session ID! */
       data->state.sessionage++;            /* increase general age */
       check->age = data->state.sessionage; /* set this as used in this age */
@@ -529,6 +548,9 @@ static int Kill_Single_Session(struct curl_ssl_session *session)
     SSL_SESSION_free(session->sessionid);
     session->sessionid=NULL;
     session->age = 0; /* fresh */
+
+    Curl_free_ssl_config(&session->ssl_config);
+
     free(session->name);
     session->name = NULL; /* no name */
 
@@ -620,6 +642,8 @@ static int Store_SSL_Session(struct connectdata *conn)
   store->name = strdup(conn->name);       /* clone host name */
   store->remote_port = conn->remote_port; /* port number */
 
+  Curl_clone_ssl_config(&conn->ssl_config, &store->ssl_config);
+
   return 0;
 }
 
@@ -673,6 +697,7 @@ static int Curl_ASN1_UTCTIME_output(struct connectdata *conn,
 #endif  
 
 /* ====================================================== */
+#ifdef USE_SSLEAY
 static int
 cert_hostcheck(const char *certname, const char *hostname)
 {
@@ -685,7 +710,7 @@ cert_hostcheck(const char *certname, const char *hostname)
      !strlen(hostname)) /* sanity check */
     return 0;
 
-  if(strequal(certname, hostname)) /* trivial case */
+  if(curl_strequal(certname, hostname)) /* trivial case */
     return 1;
 
   certdomain = certname + 1;
@@ -701,7 +726,7 @@ cert_hostcheck(const char *certname, const char *hostname)
   if(tmp) {
     /* ok the certname's domain matches the hostname, let's check that it's a
        tail-match */
-    if(strequal(tmp, certdomain))
+    if(curl_strequal(tmp, certdomain))
       /* looks like a match. Just check we havent swallowed a '.' */
       return tmp == strchr(hostname, '.');
     else
@@ -709,6 +734,7 @@ cert_hostcheck(const char *certname, const char *hostname)
   }
   return 0;
 }
+#endif
 
 /* ====================================================== */
 CURLcode
@@ -719,6 +745,7 @@ Curl_SSLConnect(struct connectdata *conn)
 #ifdef USE_SSLEAY
   struct SessionHandle *data = conn->data;
   int err;
+  int what;
   char * str;
   SSL_METHOD *req_method;
   SSL_SESSION *ssl_sessionid=NULL;
@@ -787,7 +814,7 @@ Curl_SSLConnect(struct connectdata *conn)
         !SSL_CTX_load_verify_locations(conn->ssl.ctx,
                                        data->set.ssl.CAfile,
                                        data->set.ssl.CApath)) {
-      failf(data,"error setting cerficate verify locations");
+      failf(data,"error setting certificate verify locations");
       return CURLE_SSL_CACERT;
     }
   }
@@ -816,26 +843,11 @@ Curl_SSLConnect(struct connectdata *conn)
   SSL_set_fd(conn->ssl.handle, conn->firstsocket);
 
   do {
-    int what;
     fd_set writefd;
     fd_set readfd;
     struct timeval interval;
     long timeout_ms;
 
-    err = SSL_connect(conn->ssl.handle);
-
-    what = SSL_get_error(conn->ssl.handle, err);
-
-    FD_ZERO(&writefd);
-    FD_ZERO(&readfd);
-
-    if(SSL_ERROR_WANT_READ == what)
-      FD_SET(conn->firstsocket, &readfd);
-    else if(SSL_ERROR_WANT_WRITE == what)
-      FD_SET(conn->firstsocket, &writefd);
-    else
-      break; /* untreated error */
-
     /* Find out if any timeout is set. If not, use 300 seconds.
        Otherwise, figure out the most strict timeout of the two possible one
        and then how much time that has elapsed to know how much time we
@@ -870,6 +882,60 @@ Curl_SSLConnect(struct connectdata *conn)
       /* no particular time-out has been set */
       timeout_ms=300000; /* milliseconds, default to five minutes */
 
+
+    FD_ZERO(&writefd);
+    FD_ZERO(&readfd);
+
+    err = SSL_connect(conn->ssl.handle);
+
+    /* 1  is fine
+       0  is "not successful but was shut down controlled"
+       <0 is "handshake was not successful, because a fatal error occurred" */
+    if(1 != err) {
+      int detail = SSL_get_error(conn->ssl.handle, err);
+
+      if(SSL_ERROR_WANT_READ == detail)
+        FD_SET(conn->firstsocket, &readfd);
+      else if(SSL_ERROR_WANT_WRITE == detail)
+        FD_SET(conn->firstsocket, &writefd);
+      else {
+        /* untreated error */
+        char error_buffer[120]; /* OpenSSL documents that this must be at least
+                                   120 bytes long. */
+
+        detail = ERR_get_error(); /* Gets the earliest error code from the
+                                     thread's error queue and removes the
+                                     entry. */
+
+        switch(detail) {
+        case 0x1407E086:
+          /* 1407E086:
+             SSL routines:
+             SSL2_SET_CERTIFICATE:
+             certificate verify failed */
+        case 0x14090086:
+          /* 14090086:
+             SSL routines:
+             SSL3_GET_SERVER_CERTIFICATE:
+             certificate verify failed */
+          failf(data,
+                "SSL certificate problem, verify that the CA cert is OK");
+          return CURLE_SSL_CACERT;
+        default:
+          /* detail is already set to the SSL error above */
+          failf(data, "SSL: %s", ERR_error_string(detail, error_buffer));
+          /* OpenSSL 0.9.6 and later has a function named
+             ERRO_error_string_n() that takes the size of the buffer as a third
+             argument, and we should possibly switch to using that one in the
+             future. */
+          return CURLE_SSL_CONNECT_ERROR;
+        }
+      }
+    }
+    else
+      /* we have been connected fine, get out of the connect loop */
+      break;
+
     interval.tv_sec = timeout_ms/1000;
     timeout_ms -= interval.tv_sec*1000;
 
@@ -888,15 +954,6 @@ Curl_SSLConnect(struct connectdata *conn)
       break; /* get out of loop */
   } while(1);
 
-  /* 1  is fine
-     0  is "not successful but was shut down controlled"
-     <0 is "handshake was not successful, because a fatal error occurred" */
-  if (err <= 0) {
-    err = ERR_get_error(); 
-    failf(data, "SSL: %s", ERR_error_string(err, NULL));
-    return CURLE_SSL_CONNECT_ERROR;
-  }
-
   /* Informational message */
   infof (data, "SSL connection using %s\n",
          SSL_get_cipher(conn->ssl.handle));
@@ -996,11 +1053,3 @@ Curl_SSLConnect(struct connectdata *conn)
 #endif
   return retcode;
 }
-
-/*
- * local variables:
- * eval: (load-file "../curl-mode.el")
- * end:
- * vim600: fdm=marker
- * vim: et sw=2 ts=2 sts=2 tw=78
- */
