@@ -1,16 +1,16 @@
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: url.c,v 1.252 2003/01/10 16:19:32 bagder Exp $
+ * $Id: url.c,v 1.275 2003/05/12 12:45:14 bagder Exp $
  ***************************************************************************/
 
 /* -- WIN32 approved -- */
@@ -67,7 +67,7 @@
 #include <sys/select.h>
 #endif
 
-#ifdef	VMS
+#ifdef VMS
 #include <in.h>
 #include <inet.h>
 #endif
@@ -83,8 +83,12 @@
 #error "We can't compile without socket() support!"
 #endif
 
+
 #endif
 
+#ifdef HAVE_OPENSSL_ENGINE_H
+#include <openssl/engine.h>
+#endif
 #include "urldata.h"
 #include "netrc.h"
 
@@ -102,6 +106,7 @@
 #include "escape.h"
 #include "strtok.h"
 #include "share.h"
+#include "content_encoding.h"
 
 /* And now for the protocols */
 #include "ftp.h"
@@ -139,7 +144,7 @@ static bool ConnectionExists(struct SessionHandle *data,
                              struct connectdata **usethis);
 static unsigned int ConnectionStore(struct SessionHandle *data,
                                     struct connectdata *conn);
-
+static bool safe_strequal(char* str1, char* str2);
 
 #if !defined(WIN32)||defined(__CYGWIN32__)
 #ifndef RETSIGTYPE
@@ -190,6 +195,9 @@ CURLcode Curl_close(struct SessionHandle *data)
   if(data->state.auth_host)
     free(data->state.auth_host);
 
+  if(data->state.scratch)
+    free(data->state.scratch);
+
   if(data->change.proxy_alloc)
     free(data->change.proxy);
 
@@ -203,10 +211,12 @@ CURLcode Curl_close(struct SessionHandle *data)
     free(data->state.headerbuff);
 
 #ifndef CURL_DISABLE_HTTP
-  if(data->set.cookiejar)
+  if(data->set.cookiejar) {
     /* we have a "destination" for all the cookies to get dumped to */
-    Curl_cookie_output(data->cookies, data->set.cookiejar);
-    
+    if(Curl_cookie_output(data->cookies, data->set.cookiejar))
+      infof(data, "WARNING: failed to save cookies in given jar\n");
+  }
+
   Curl_cookie_cleanup(data->cookies);
 #endif
 
@@ -243,7 +253,7 @@ CURLcode Curl_open(struct SessionHandle **curl)
   if(!data)
     /* this is a very serious error */
     return CURLE_OUT_OF_MEMORY;
-  
+
   memset(data, 0, sizeof(struct SessionHandle));
 
   /* We do some initial setup here, all those fields that can't be just 0 */
@@ -259,13 +269,13 @@ CURLcode Curl_open(struct SessionHandle **curl)
   data->set.out = stdout; /* default output to stdout */
   data->set.in  = stdin;  /* default input from stdin */
   data->set.err  = stderr;  /* default stderr to stderr */
-  
+
   /* use fwrite as default function to store output */
   data->set.fwrite = (curl_write_callback)fwrite;
 
   /* use fread as default function to read input */
   data->set.fread = (curl_read_callback)fread;
-  
+
   /* set the default passwd function */
   data->set.fpasswd = my_getpass;
 
@@ -275,9 +285,10 @@ CURLcode Curl_open(struct SessionHandle **curl)
 
   data->set.httpreq = HTTPREQ_GET; /* Default HTTP request */
   data->set.ftp_use_epsv = TRUE;   /* FTP defaults to EPSV operations */
+  data->set.ftp_use_eprt = TRUE;   /* FTP defaults to EPRT operations */
 
   data->set.dns_cache_timeout = 60; /* Timeout every 60 seconds by default */
-  
+
   /* make libcurl quiet by default: */
   data->set.hide_progress = TRUE;  /* CURLOPT_NOPROGRESS changes these */
   data->progress.flags |= PGRS_HIDE;
@@ -286,14 +297,14 @@ CURLcode Curl_open(struct SessionHandle **curl)
   data->set.ssl.numsessions = 5;
 
   data->set.proxyport = 1080;
-  
+
   data->set.proxytype = CURLPROXY_HTTP; /* defaults to HTTP proxy */
 
   /* create an array with connection data struct pointers */
   data->state.numconnects = 5; /* hard-coded right now */
   data->state.connects = (struct connectdata **)
     malloc(sizeof(struct connectdata *) * data->state.numconnects);
-  
+
   if(!data->state.connects) {
     free(data);
     return CURLE_OUT_OF_MEMORY;
@@ -329,7 +340,7 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
   case CURLOPT_DNS_CACHE_TIMEOUT:
     data->set.dns_cache_timeout = va_arg(param, int);
     break;
-  case CURLOPT_DNS_USE_GLOBAL_CACHE: 
+  case CURLOPT_DNS_USE_GLOBAL_CACHE:
     {
       int use_cache = va_arg(param, int);
       if (use_cache) {
@@ -375,12 +386,19 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
           Curl_disconnect(data->state.connects[i]);
       }
       if(newconnects) {
+        int i;
         newptr= (struct connectdata **)
           realloc(data->state.connects,
                   sizeof(struct connectdata *) * newconnects);
         if(!newptr)
           /* we closed a few connections in vain, but so what? */
           return CURLE_OUT_OF_MEMORY;
+
+        /* nullify the newly added pointers */
+        for(i=data->state.numconnects; i<newconnects; i++) {
+          newptr[i] = NULL;
+        }
+
         data->state.connects = newptr;
         data->state.numconnects = newconnects;
       }
@@ -484,6 +502,14 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
      */
     data->set.http_follow_location = va_arg(param, long)?TRUE:FALSE;
     break;
+  case CURLOPT_UNRESTRICTED_AUTH:
+    /*
+     * Send authentication (user+password) when following locations, even when
+     * hostname changed.
+     */
+    data->set.http_disable_hostname_check_before_authentication =
+      va_arg(param, long)?TRUE:FALSE;
+    break;
   case CURLOPT_HTTP_VERSION:
     /*
      * This sets a requested HTTP version to be used. The value is one of
@@ -610,10 +636,14 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
     data->set.ftp_use_port = data->set.ftpport?1:0;
     break;
 
+  case CURLOPT_FTP_USE_EPRT:
+    data->set.ftp_use_eprt = va_arg(param, long)?TRUE:FALSE;
+    break;
+
   case CURLOPT_FTP_USE_EPSV:
     data->set.ftp_use_epsv = va_arg(param, long)?TRUE:FALSE;
     break;
-
+    
   case CURLOPT_HTTPHEADER:
     /*
      * Set a list with HTTP headers to use (or replace internals with)
@@ -796,8 +826,16 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
   case CURLOPT_ENCODING:
     /*
      * String to use at the value of Accept-Encoding header. 08/28/02 jhrg
+     *
+     * If the encoding is set to "" we use an Accept-Encoding header that
+     * encompasses all the encodings we support.
+     * If the encoding is set to NULL we don't send an Accept-Encoding header
+     * and ignore an received Content-Encoding header.
+     *
      */
     data->set.encoding = va_arg(param, char *);
+    if(data->set.encoding && !*data->set.encoding)
+      data->set.encoding = (char*)ALL_CONTENT_ENCODINGS;
     break;
 
   case CURLOPT_USERPWD:
@@ -808,7 +846,7 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
     break;
   case CURLOPT_POSTQUOTE:
     /*
-     * List of RAW FTP commands to use after a transfer 
+     * List of RAW FTP commands to use after a transfer
      */
     data->set.postquote = va_arg(param, struct curl_slist *);
     break;
@@ -820,7 +858,7 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
     break;
   case CURLOPT_QUOTE:
     /*
-     * List of RAW FTP commands to use before a transfer 
+     * List of RAW FTP commands to use before a transfer
      */
     data->set.quote = va_arg(param, struct curl_slist *);
     break;
@@ -1075,10 +1113,33 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
       struct Curl_share *set;
       set = va_arg(param, struct Curl_share *);
       if(data->share)
+      {
+        Curl_share_lock(data, CURL_LOCK_DATA_SHARE, CURL_LOCK_ACCESS_SINGLE);
+
+        /* checking the dns cache stuff */
+        if(data->share->hostcache == data->hostcache)
+        {
+          data->hostcache = NULL;
+        }
+
         data->share->dirty--;
 
+        Curl_share_unlock(data, CURL_LOCK_DATA_SHARE);
+      }
+
       data->share = set;
+
+      Curl_share_lock(data, CURL_LOCK_DATA_SHARE, CURL_LOCK_ACCESS_SINGLE);
+
       data->share->dirty++;
+
+      if( data->hostcache )
+      {
+        Curl_hash_destroy(data->hostcache);
+        data->hostcache = data->share->hostcache;
+      }
+
+      Curl_share_unlock(data, CURL_LOCK_DATA_SHARE);
     }
     break;
 
@@ -1102,7 +1163,7 @@ CURLcode Curl_setopt(struct SessionHandle *data, CURLoption option, ...)
      */
     data->set.http200aliases = va_arg(param, struct curl_slist *);
     break;
-      
+
   default:
     /* unknown tag and its companion, just ignore: */
     return CURLE_FAILED_INIT; /* correct this */
@@ -1170,10 +1231,14 @@ CURLcode Curl_disconnect(struct connectdata *conn)
     free(conn->allocptr.cookie);
   if(conn->allocptr.host)
     free(conn->allocptr.host);
+  if(conn->allocptr.cookiehost)
+    free(conn->allocptr.cookiehost);
 
   if(conn->proxyhost)
     free(conn->proxyhost);
 
+  Curl_free_ssl_config(&conn->ssl_config);
+
   free(conn); /* free all the connection oriented data */
 
   return CURLE_OK;
@@ -1184,24 +1249,24 @@ CURLcode Curl_disconnect(struct connectdata *conn)
  * be dead. Most commonly this happens when the server has closed the
  * connection due to inactivity.
  */
-static bool SocketIsDead(int sock) 
-{ 
-  int sval; 
-  bool ret_val = TRUE; 
-  fd_set check_set; 
-  struct timeval to; 
+static bool SocketIsDead(int sock)
+{
+  int sval;
+  bool ret_val = TRUE;
+  fd_set check_set;
+  struct timeval to;
 
-  FD_ZERO(&check_set); 
-  FD_SET(sock,&check_set); 
+  FD_ZERO(&check_set);
+  FD_SET(sock,&check_set);
 
-  to.tv_sec = 0; 
-  to.tv_usec = 0; 
+  to.tv_sec = 0;
+  to.tv_usec = 0;
 
   sval = select(sock + 1, &check_set, 0, 0, &to);
   if(sval == 0)
     /* timeout */
     ret_val = FALSE;
-  
+
   return ret_val;
 }
 
@@ -1228,6 +1293,11 @@ ConnectionExists(struct SessionHandle *data,
     if(!check)
       /* NULL pointer means not filled-in entry */
       continue;
+
+    if((needle->protocol&PROT_SSL) != (check->protocol&PROT_SSL))
+      /* don't do mixed SSL and non-SSL connections */
+      continue;
+
     if(!needle->bits.httpproxy || needle->protocol&PROT_SSL) {
       /* The requested connection does not use a HTTP proxy or it
          uses SSL. */
@@ -1240,7 +1310,15 @@ ConnectionExists(struct SessionHandle *data,
       if(strequal(needle->protostr, check->protostr) &&
          strequal(needle->name, check->name) &&
          (needle->remote_port == check->remote_port) ) {
-        if(strequal(needle->protostr, "FTP")) {
+        if(needle->protocol & PROT_SSL) {
+          /* This is SSL, verify that we're using the same
+             ssl options as well */
+          if(!Curl_ssl_config_matches(&needle->ssl_config,
+                                      &check->ssl_config)) {
+            continue;
+          }
+        }
+        if(needle->protocol & PROT_FTP) {
           /* This is FTP, verify that we're using the same name and
              password as well */
           if(!strequal(needle->data->state.user, check->proto.ftp->user) ||
@@ -1303,7 +1381,7 @@ ConnectionKillOne(struct SessionHandle *data)
 
   for(i=0; i< data->state.numconnects; i++) {
     conn = data->state.connects[i];
-    
+
     if(!conn)
       continue;
 
@@ -1390,8 +1468,8 @@ static int handleSock5Proxy(
     int sock)
 {
   unsigned char socksreq[600]; /* room for large user/pw (255 max each) */
-  int actualread;
-  int written;
+  ssize_t actualread;
+  ssize_t written;
   CURLcode result;
 
   Curl_nonblock(sock, FALSE);
@@ -1480,7 +1558,7 @@ static int handleSock5Proxy(
               " that the SOCKS5 server wanted a username/password, since none"
               " was supplied to the server on this connection.)");
       }
-      else {            
+      else {
         failf(conn->data, "No authentication method was acceptable.");
       }
       return 1;
@@ -1497,7 +1575,7 @@ static int handleSock5Proxy(
   socksreq[1] = 1; /* connect */
   socksreq[2] = 0; /* must be zero */
   socksreq[3] = 1; /* IPv4 = 1 */
-    
+
   {
 #ifndef ENABLE_IPV6
     struct Curl_dns_entry *dns;
@@ -1515,7 +1593,7 @@ static int handleSock5Proxy(
       socksreq[6] = ((char*)hp->h_addr_list[0])[2];
       socksreq[7] = ((char*)hp->h_addr_list[0])[3];
 
-      Curl_resolv_unlock(dns); /* not used anymore from now on */
+      Curl_resolv_unlock(conn->data, dns); /* not used anymore from now on */
     }
     else {
       failf(conn->data, "Failed to resolve \"%s\" for SOCKS5 connect.",
@@ -1611,7 +1689,7 @@ static CURLcode ConnectPlease(struct connectdata *conn,
     }
     else {
       failf(conn->data, "unknown proxytype option given");
-      return CURLE_COULDNT_CONNECT; 
+      return CURLE_COULDNT_CONNECT;
     }
   }
 
@@ -1684,7 +1762,7 @@ CURLcode Curl_protocol_connect(struct connectdata *conn,
 {
   struct SessionHandle *data = conn->data;
   CURLcode result=CURLE_OK;
-  
+
   if(conn->bits.tcpconnect)
     /* We already are connected, get back. This may happen when the connect
        worked fine in the first call, like when we connect to a local server
@@ -1715,7 +1793,6 @@ static CURLcode CreateConnection(struct SessionHandle *data,
                                  struct connectdata **in_connect)
 {
   char *tmp;
-  char *buf;
   CURLcode result=CURLE_OK;
   char resumerange[40]="";
   struct connectdata *conn;
@@ -1769,42 +1846,32 @@ static CURLcode CreateConnection(struct SessionHandle *data,
   conn->bits.httpproxy = (data->change.proxy && *data->change.proxy &&
                           (data->set.proxytype == CURLPROXY_HTTP))?
     TRUE:FALSE; /* http proxy or not */
-  conn->bits.use_range = data->set.set_range?TRUE:FALSE; /* range status */
-  conn->range = data->set.set_range;              /* clone the range setting */
-  conn->resume_from = data->set.set_resume_from;   /* inherite resume_from */
 
   /* Default protocol-independent behavior doesn't support persistant
      connections, so we set this to force-close. Protocols that support
      this need to set this to FALSE in their "curl_do" functions. */
   conn->bits.close = TRUE;
 
-  /* inherite initial knowledge from the data struct */
-  conn->bits.user_passwd = data->set.userpwd?1:0;
-  conn->bits.proxy_user_passwd = data->set.proxyuserpwd?1:0;
-
   /* maxdownload must be -1 on init, as 0 is a valid value! */
   conn->maxdownload = -1;  /* might have been used previously! */
 
   /* Store creation time to help future close decision making */
   conn->created = Curl_tvnow();
 
+  conn->bits.use_range = data->set.set_range?TRUE:FALSE; /* range status */
+  conn->range = data->set.set_range;              /* clone the range setting */
+  conn->resume_from = data->set.set_resume_from;   /* inherite resume_from */
+
   /* Set the start time temporary to this creation time to allow easier
      timeout checks before the transfer has started for real. The start time
      is later set "for real" using Curl_pgrsStartNow(). */
-  conn->data->progress.start = conn->created; 
+  conn->data->progress.start = conn->created;
 
-  conn->bits.upload_chunky =
-    ((conn->protocol&PROT_HTTP) &&
-     data->set.upload &&
-     (data->set.infilesize == -1) &&
-     (data->set.httpversion != CURL_HTTP_VERSION_1_0))?
-    /* HTTP, upload, unknown file size and not HTTP 1.0 */
-    TRUE:
-  /* else, no chunky upload */
-  FALSE;
+  conn->bits.user_passwd = data->set.userpwd?1:0;
+  conn->bits.proxy_user_passwd = data->set.proxyuserpwd?1:0;
 
-  conn->fread = data->set.fread;
-  conn->fread_in = data->set.in;
+  /* This initing continues below, see the comment "Continue connectdata
+   * initialization here" */
 
   /***********************************************************
    * We need to allocate memory to store the path in. We get the size of the
@@ -1815,7 +1882,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
   urllen=strlen(data->change.url);
   if(urllen < LEAST_PATH_ALLOC)
     urllen=LEAST_PATH_ALLOC;
-  
+
   conn->path=(char *)malloc(urllen);
   if(NULL == conn->path)
     return CURLE_OUT_OF_MEMORY; /* really bad error */
@@ -1842,14 +1909,14 @@ static CURLcode CreateConnection(struct SessionHandle *data,
       char *ptr=strchr(conn->path, '/');
       if(ptr) {
         /* there was a slash present
-           
+
            RFC1738 (section 3.1, page 5) says:
- 
+
            The rest of the locator consists of data specific to the scheme,
            and is known as the "url-path". It supplies the details of how the
            specified resource can be accessed. Note that the "/" between the
            host (or port) and the url-path is NOT part of the url-path.
- 
+
            As most agents use file://localhost/foo to get '/foo' although the
            slash preceeding foo is a separator and not a slash for the path,
            a URL as file://localhost//foo must be valid as well, to refer to
@@ -1859,8 +1926,8 @@ static CURLcode CreateConnection(struct SessionHandle *data,
         if(ptr[1] && ('/' == ptr[1]))
           /* if there was two slashes, we skip the first one as that is then
              used truly as a separator */
-          ptr++; 
-        
+          ptr++;
+
         strcpy(conn->path, ptr);
       }
     }
@@ -1871,16 +1938,20 @@ static CURLcode CreateConnection(struct SessionHandle *data,
     /* Set default host and default path */
     strcpy(conn->gname, "curl.haxx.se");
     strcpy(conn->path, "/");
-
+    /* We need to search for '/' OR '?' - whichever comes first after host
+     * name but before the path. We need to change that to handle things like
+     * http://example.com?param= (notice the missing '/'). Later we'll insert
+     * that missing slash at the beginning of the path.
+     */
     if (2 > sscanf(data->change.url,
-                   "%64[^\n:]://%512[^\n/]%[^\n]",
+                   "%64[^\n:]://%512[^\n/?]%[^\n]",
                    conn->protostr, conn->gname, conn->path)) {
-      
+
       /*
        * The URL was badly formatted, let's try the browser-style _without_
        * protocol specified like 'http://'.
        */
-      if((1 > sscanf(data->change.url, "%512[^\n/]%[^\n]",
+      if((1 > sscanf(data->change.url, "%512[^\n/?]%[^\n]",
                      conn->gname, conn->path)) ) {
         /*
          * We couldn't even get this format.
@@ -1922,7 +1993,17 @@ static CURLcode CreateConnection(struct SessionHandle *data,
     }
   }
 
-  buf = data->state.buffer; /* this is our buffer */
+  /* If the URL is malformatted (missing a '/' after hostname before path) we
+   * insert a slash here. The only letter except '/' we accept to start a path
+   * is '?'.
+   */
+  if(conn->path[0] == '?') {
+    /* We need this function to deal with overlapping memory areas. We know
+       that the memory area 'path' points to is 'urllen' bytes big and that
+       is bigger than the path. Use +1 to move the zero byte too. */
+    memmove(&conn->path[1], conn->path, strlen(conn->path)+1);
+    conn->path[0] = '/';
+  }
 
   /*
    * So if the URL was A://B/C,
@@ -1961,7 +2042,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
   }
 
   /*************************************************************
-   * Set a few convenience pointers 
+   * Set a few convenience pointers
    *************************************************************/
   conn->name = conn->gname;
   conn->ppath = conn->path;
@@ -2046,7 +2127,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
          * environment (cgi or php), this environment variable can
          * be controlled by the web server user by setting the
          * http header 'Proxy:' to some value.
-         * 
+         *
          * This can cause 'internal' http/ftp requests to be
          * arbitrarily redirected by any external attacker.
          */
@@ -2348,7 +2429,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
     /* this is a RFC2732-style specified IP-address */
     conn->bits.ipv6_ip = TRUE;
 
-    conn->name++; /* pass the starting bracket */ 
+    conn->name++; /* pass the starting bracket */
     conn->hostname++;
     tmp = strchr(conn->name, ']');
     *tmp = 0; /* zero terminate */
@@ -2459,7 +2540,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
 
   data->state.user[0] =0;   /* to make everything well-defined */
   data->state.passwd[0]=0;
-  
+
   if (conn->protocol & (PROT_FTP|PROT_HTTP)) {
     /* This is a FTP or HTTP URL, we will now try to extract the possible
      * user+password pair in a string like:
@@ -2491,9 +2572,6 @@ static CURLcode CreateConnection(struct SessionHandle *data,
           /* no name given, get the password only */
           sscanf(userpass, ":%127[^@]", data->state.passwd);
 
-        /* we have set the password */
-        data->state.passwdgiven = TRUE;
-
         if(data->state.user[0]) {
           char *newname=curl_unescape(data->state.user, 0);
           if(strlen(newname) < sizeof(data->state.user)) {
@@ -2510,6 +2588,9 @@ static CURLcode CreateConnection(struct SessionHandle *data,
             strcpy(data->state.passwd, newpasswd);
           }
           free(newpasswd);
+
+          /* we have set the password */
+          data->state.passwdgiven = TRUE;
         }
       }
     }
@@ -2538,15 +2619,18 @@ static CURLcode CreateConnection(struct SessionHandle *data,
       sscanf(data->set.userpwd+1, "%127[^\n]", data->state.passwd);
   }
 
-  if (data->set.use_netrc != CURL_NETRC_IGNORED &&
+  if ((data->set.use_netrc != CURL_NETRC_IGNORED) &&
       !data->state.passwdgiven) {  /* need passwd */
     if(Curl_parsenetrc(conn->hostname,
                        data->state.user,
                        data->state.passwd)) {
       infof(data, "Couldn't find host %s in the .netrc file, using defaults",
             conn->hostname);
-    } else
+    }
+    else {
       conn->bits.user_passwd = 1; /* enable user+password */
+      data->state.passwdgiven = TRUE;
+    }
   }
 
   /* if we have a user but no password, ask for one */
@@ -2576,6 +2660,11 @@ static CURLcode CreateConnection(struct SessionHandle *data,
    * new one.
    *************************************************************/
 
+  /* get a cloned copy of the SSL config situation stored in the 
+     connection struct */
+  if(!Curl_clone_ssl_config(&data->set.ssl, &conn->ssl_config))
+    return CURLE_OUT_OF_MEMORY;
+
   /* reuse_fresh is set TRUE if we are told to use a fresh connection
      by force */
   if(!data->set.reuse_fresh &&
@@ -2591,8 +2680,18 @@ static CURLcode CreateConnection(struct SessionHandle *data,
     char *ppath = old_conn->ppath; /* this is the modified path pointer */
     if(old_conn->proxyhost)
       free(old_conn->proxyhost);
+
+    /* free the SSL config struct from this connection struct as this was
+       allocated in vain and is targeted for destruction */
+    Curl_free_ssl_config(&conn->ssl_config);
+
     conn = conn_temp;        /* use this connection from now on */
 
+    /* get the user+password information from the old_conn struct since it may
+     * be new for this request even when we re-use an existing connection */
+    conn->bits.user_passwd = old_conn->bits.user_passwd;
+    conn->bits.proxy_user_passwd = conn->bits.proxy_user_passwd;
+
     /* If we speak over a proxy, we need to copy the host name too, as it
        might be another remote host even when re-using a connection */
     strcpy(conn->gname, old_conn->gname); /* safe strcpy() */
@@ -2605,7 +2704,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
 
     /* 'path' points to the allocated data, 'ppath' may have been advanced
        to point somewhere within the 'path' area. */
-    conn->path = path; 
+    conn->path = path;
     conn->ppath = ppath;
 
     /* re-use init */
@@ -2623,9 +2722,9 @@ static CURLcode CreateConnection(struct SessionHandle *data,
     conn->resume_from = data->set.set_resume_from;
     if (conn->resume_from) {
         snprintf(resumerange, sizeof(resumerange), "%d-", conn->resume_from);
-        if (conn->bits.rangestringalloc == TRUE) 
+        if (conn->bits.rangestringalloc == TRUE)
             free(conn->range);
-        
+
         /* tell ourselves to fetch this range */
         conn->range = strdup(resumerange);
         conn->bits.use_range = TRUE;        /* enable range download */
@@ -2637,7 +2736,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
       conn->bits.rangestringalloc = TRUE; /* mark range string allocated */
       conn->bits.use_range = TRUE;        /* enable range download */
     }
-    
+
     *in_connect = conn;      /* return this instead! */
 
     infof(data, "Re-using existing connection! (#%d)\n", conn->connectindex);
@@ -2650,6 +2749,23 @@ static CURLcode CreateConnection(struct SessionHandle *data,
     ConnectionStore(data, conn);
   }
 
+  /* Continue connectdata initialization here.
+   *
+   * Inherit the proper values from the urldata struct AFTER we have arranged
+   * the persistant conncetion stuff */
+  conn->fread = data->set.fread;
+  conn->fread_in = data->set.in;
+
+  conn->bits.upload_chunky =
+    ((conn->protocol&PROT_HTTP) &&
+     data->set.upload &&
+     (data->set.infilesize == -1) &&
+     (data->set.httpversion != CURL_HTTP_VERSION_1_0))?
+    /* HTTP, upload, unknown file size and not HTTP 1.0 */
+    TRUE:
+  /* else, no chunky upload */
+  FALSE;
+
   /*************************************************************
    * Set timeout if that is being used
    *************************************************************/
@@ -2811,7 +2927,7 @@ static CURLcode CreateConnection(struct SessionHandle *data,
 
   conn->bytecount = 0;
   conn->headerbytecount = 0;
-  
+
   if(-1 == conn->firstsocket) {
     bool connected;
 
@@ -2895,7 +3011,7 @@ CURLcode Curl_done(struct connectdata *conn)
   }
 
   if(conn->connect_addr)
-    Curl_resolv_unlock(conn->connect_addr); /* done with this */
+    Curl_resolv_unlock(conn->data, conn->connect_addr); /* done with this */
 
 #if defined(MALLOCDEBUG) && defined(AGGRESIVE_TEST)
   /* scan for DNS cache entries still marked as in use */
@@ -2913,7 +3029,7 @@ CURLcode Curl_done(struct connectdata *conn)
 
   /* if data->set.reuse_forbid is TRUE, it means the libcurl client has
      forced us to close this no matter what we think.
-    
+
      if conn->bits.close is TRUE, it means that the connection should be
      closed in spite of all our efforts to be nice, due to protocol
      restrictions in our or the server's end */
@@ -2939,7 +3055,7 @@ CURLcode Curl_do(struct connectdata **connp)
     result = conn->curl_do(conn);
 
     /* This was formerly done in transfer.c, but we better do it here */
-    
+
     if((CURLE_SEND_ERROR == result) && conn->bits.reuse) {
       /* This was a re-use of a connection and we got a write error in the
        * DO-phase. Then we DISCONNECT this connection and have another attempt
@@ -2973,10 +3089,88 @@ CURLcode Curl_do_more(struct connectdata *conn)
   return result;
 }
 
-/*
- * local variables:
- * eval: (load-file "../curl-mode.el")
- * end:
- * vim600: fdm=marker
- * vim: et sw=2 ts=2 sts=2 tw=78
- */
+static bool safe_strequal(char* str1, char* str2)
+{
+  if(str1 && str2)
+    /* both pointers point to something then compare them */
+    return strequal(str1, str2);
+  else
+    /* if both pointers are NULL then treat them as equal */
+    return (!str1 && !str2);
+}
+
+bool
+Curl_ssl_config_matches(struct ssl_config_data* data,
+                        struct ssl_config_data* needle)
+{
+  if((data->version == needle->version) &&
+     (data->verifypeer == needle->verifypeer) &&
+     (data->verifyhost == needle->verifyhost) &&
+     safe_strequal(data->CApath, needle->CApath) &&
+     safe_strequal(data->CAfile, needle->CAfile) &&
+     safe_strequal(data->random_file, needle->random_file) &&
+     safe_strequal(data->egdsocket, needle->egdsocket) &&
+     safe_strequal(data->cipher_list, needle->cipher_list))
+    return TRUE;
+
+  return FALSE;
+}
+
+bool
+Curl_clone_ssl_config(struct ssl_config_data *source,
+                      struct ssl_config_data *dest)
+{
+  dest->verifyhost = source->verifyhost;
+  dest->verifypeer = source->verifypeer;
+  dest->version = source->version;
+
+  if(source->CAfile) {
+    dest->CAfile = strdup(source->CAfile);
+    if(!dest->CAfile)
+      return FALSE;
+  }
+
+  if(source->CApath) {
+    dest->CApath = strdup(source->CApath);
+    if(!dest->CApath)
+      return FALSE;
+  }
+
+  if(source->cipher_list) {
+    dest->cipher_list = strdup(source->cipher_list);
+    if(!dest->cipher_list)
+      return FALSE;
+  }
+
+  if(source->egdsocket) {
+    dest->egdsocket = strdup(source->egdsocket);
+    if(!dest->egdsocket)
+      return FALSE;
+  }
+
+  if(source->random_file) {
+    dest->random_file = strdup(source->random_file);
+    if(!dest->random_file)
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
+void Curl_free_ssl_config(struct ssl_config_data* sslc)
+{
+  if(sslc->CAfile)
+    free(sslc->CAfile);
+
+  if(sslc->CApath)
+    free(sslc->CApath);
+
+  if(sslc->cipher_list)
+    free(sslc->cipher_list);
+
+  if(sslc->egdsocket)
+    free(sslc->egdsocket);
+
+  if(sslc->random_file)
+    free(sslc->random_file);
+}
