@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,11 +20,9 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.15 2002/12/04 11:06:17 bagder Exp $
+ * $Id: setup.h,v 1.17 2003/04/15 14:01:57 bagder Exp $
  ***************************************************************************/
 
-#include <stdio.h>
-
 #if !defined(WIN32) && defined(__WIN32__)
 /* Borland fix */
 #define WIN32
@@ -50,6 +48,15 @@
 #endif
 #endif
 
+#ifdef MALLOCDEBUG
+/* This is an ugly hack for MALLOCDEBUG conditions only. We need to include
+   the file here, since it might set the _FILE_OFFSET_BITS define, which must
+   be set BEFORE all normal system headers. */
+#include "../lib/setup.h"
+#endif
+
+#include <stdio.h>
+
 #ifndef OS
 #define OS "unknown"
 #endif
