@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: runtests.pl,v 1.69 2003/01/09 11:26:57 bagder Exp $
+# $Id: runtests.pl,v 1.80 2003/04/30 20:28:49 bagder Exp $
 #
 # Main curl test script, in perl to run on more platforms
 #
@@ -66,7 +66,8 @@ my $checkstunnel = &checkstunnel;
 
 my $ssl_version; # set if libcurl is built with SSL support
 
-my $skipped=0; # number of tests skipped; reported in main loop
+my $skipped=0;  # number of tests skipped; reported in main loop
+my $problems=0; # number of tests that didn't run due to run-time problems
 
 #######################################################################
 # variables the command line options may set
@@ -82,6 +83,8 @@ my $listonly;     # only list the tests
 
 my $pwd;          # current working directory
 
+my %run;	  # running server
+
 chomp($pwd = `pwd`);
 
 # enable memory debugging if curl is compiled with it
@@ -94,7 +97,7 @@ $ENV{'HOME'}=$pwd;
 sub serverpid {
     my $PIDFILE = $_[0];
     open(PFILE, "<$PIDFILE");
-    my $PID=<PFILE>;
+    my $PID=0+<PFILE>;
     close(PFILE);
     return $PID;
 }
@@ -112,10 +115,10 @@ sub stopserver {
         unlink $PIDFILE; # server is killed
 
         if($res && $verbose) {
-            print "Test server pid $PID signalled to die\n";
+            print "RUN: Test server pid $PID signalled to die\n";
         }
         elsif($verbose) {
-            print "Test server pid $PID didn't exist\n";
+            print "RUN: Test server pid $PID didn't exist\n";
         }
     }
 }
@@ -157,42 +160,36 @@ sub runhttpserver {
     $pid = checkserver ($HTTPPIDFILE);
 
     # verify if our/any server is running on this port
-    my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
+    my $cmd = "$CURL -o log/verifiedserver --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null";
+    print "CMD; $cmd\n" if ($verbose);
+    my $res = system($cmd);
 
-    if ( $data =~ /WE ROOLZ(: |)(\d*)/ ) {
-        if($2) {
-            $pid = 0+$2;
-        }
+    $res >>= 8; # rotate the result
+    my $data;
 
-        if(!$pid) {
-            print "Test server already running with unknown pid! Use it...\n";
-            return;
-        }
+    print "RUN: curl command returned $res\n" if ($verbose);
 
-        if($verbose) {
-            print "Test server already running with pid $pid, killing it...\n";
-        }
+    open(FILE, "<log/verifiedserver");
+    my @file=<FILE>;
+    close(FILE);
+    $data=$file[0]; # first line
+
+    if ( $data =~ /WE ROOLZ: (\d+)/ ) {
+        $pid = 0+$1;
     }
-    elsif($data ne "") {
-        print "GOT: $data\n";
-        print "An alien HTTP server is running on port $HOSTPORT\n",
-        "Edit runtests.pl to use another port and rerun the test script\n";
-        exit;
-    }
-    else {
-        if($verbose) {
-            print "No server running, start it\n";
-        }
+    elsif($data) {
+        print "RUN: Unknown HTTP server is running on port $HOSTPORT\n";
+        return 2;
     }
 
     if($pid > 0) {
         my $res = kill (9, $pid); # die!
         if(!$res) {
-            print "Failed to kill our HTTP test server, do it manually and",
+            print "RUN: Failed to kill test HTTP server, do it manually and",
             " restart the tests.\n";
             exit;
         }
-        sleep(2);
+        sleep(1);
     }
 
     my $flag=$debugprotocol?"-v ":"";
@@ -202,6 +199,30 @@ sub runhttpserver {
         print "CMD: $cmd\n";
     }
 
+    my $verified;
+    for(1 .. 5) {
+        # verify that our server is up and running:
+        my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
+
+        if ( $data !~ /WE ROOLZ: (\d+)/ ) {
+            sleep(1);
+            next;
+        }
+        else {
+            $verified = 1;
+            last;
+        }
+    }
+    if(!$verified) {
+        print STDERR "RUN: failed to start our HTTP server\n";
+        return 1;
+    }
+
+    if($verbose) {
+        print "RUN: HTTP server is now verified to be our server\n";
+    }
+
+    return 0;
 }
 
 #######################################################################
@@ -216,17 +237,18 @@ sub runhttpsserver {
     if($PID > 0) {
         # kill previous stunnel!
         if($verbose) {
-            print "kills off running stunnel at $PID\n";
+            print "RUN: kills off running stunnel at $PID\n";
         }
         stopserver($HTTPSPIDFILE);
     }
 
     my $flag=$debugprotocol?"-v ":"";
-    my $cmd="$perl $srcdir/httpsserver.pl $flag -r $HOSTPORT $HTTPSPORT &";
+    my $cmd="$perl $srcdir/httpsserver.pl $flag -d $srcdir -r $HOSTPORT $HTTPSPORT &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
     }
+    sleep(1);
 }
 
 #######################################################################
@@ -240,42 +262,75 @@ sub runftpserver {
     my $pid = checkserver ($FTPPIDFILE );
 
     if ($pid <= 0) {
-        my $flag=$debugprotocol?"-v ":"";
+        print "RUN: Check port $FTPPORT for our own FTP server\n"
+            if ($verbose);
 
-        # verify that our server is NOT running on this port:
-        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
 
-        if ( $data =~ /WE ROOLZ/ ) {
-            print "A previous FTP server session is already running and we ",
-            "can't kill it!\n";
-            exit;
+        my $time=time();
+        # check if this is our server running on this port:
+        my $data=`$CURL -m4 --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
+
+        # if this took more than 2 secs, we assume it "hung" on a weird server
+        my $took = time()-$time;
+        
+        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
+            # this is our test server with a known pid!
+            $pid = $1;
         }
-
-        my $cmd="$perl $srcdir/ftpserver.pl $flag $FTPPORT &";
-        if($verbose) {
-            print "CMD: $cmd\n";
-        }
-        system($cmd);
-    }
-    else {
-        if($verbose) {
-            print "ftpd ($pid) is already running\n";
-        }
-
-        # verify that our server is one one running on this port:
-        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
-
-        if ( $data !~ /WE ROOLZ/ ) {
-            print "Another FTP server is running on port $FTPPORT\n",
-            "Edit runtests.pl to use another FTP port and rerun the ",
-            "test script\n";
-            exit;
-        }
-
-        if($verbose) {
-            print "The running FTP server has been verified to be our server\n";
+        else {
+            if($data || ($took > 2)) {
+                # this is not a known server
+                print "RUN: Unknown server on our favourite port: $FTPPORT\n";
+                return 1;
+            }
         }
     }
+
+    if($pid > 0) {
+        print "RUN: Killing a previous server using pid $pid\n" if($verbose);
+        my $res = kill (9, $pid); # die!
+        if(!$res) {
+            print "RUN: Failed to kill our FTP test server, do it manually and",
+            " restart the tests.\n";
+            exit;
+        }
+        sleep(1);
+    }
+    
+    # now (re-)start our server:
+    my $flag=$debugprotocol?"-v ":"";
+    my $cmd="$perl $srcdir/ftpserver.pl $flag $FTPPORT &";
+    if($verbose) {
+        print "CMD: $cmd\n";
+    }
+    system($cmd);
+
+    my $verified;
+    for(1 .. 5) {
+        # verify that our server is up and running:
+        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
+
+        if ( $data !~ /WE ROOLZ: (\d+)/ ) {
+            if($verbose) {
+                print STDERR "RUN: Retrying FTP server existance in 1 sec\n";
+            }
+            sleep(1);
+            next;
+        }
+        else {
+            $verified = 1;
+            last;
+        }
+    }
+    if(!$verified) {
+        die "RUN: failed to start our FTP server\n";
+    }
+
+    if($verbose) {
+        print "RUN: FTP server is now verified to be our server\n";
+    }
+
+    return 0;
 }
 
 #######################################################################
@@ -296,11 +351,12 @@ sub runftpsserver {
     }
 
     my $flag=$debugprotocol?"-v ":"";
-    my $cmd="$perl $srcdir/ftpsserver.pl $flag -r $FTPPORT $FTPSPORT &";
+    my $cmd="$perl $srcdir/ftpsserver.pl $flag -d $srcdir -r $FTPPORT $FTPSPORT &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
     }
+    sleep(1);
 }
 
 #######################################################################
@@ -357,12 +413,18 @@ sub filteroff {
 
 sub compare {
     # filter off patterns _before_ this comparison!
-    my ($firstref, $secondref)=@_;
+    my ($subject, $firstref, $secondref)=@_;
 
     my $result = compareparts($firstref, $secondref);
 
-    if(!$short && $result) {
-        print showdiff($firstref, $secondref);
+    if($result) {
+        if(!$short) {
+            print "\n $subject FAILED:\n";
+            print showdiff($firstref, $secondref);
+        }
+        else {
+            print "FAILED\n";
+        }
     }
     return $result;
 }
@@ -440,7 +502,22 @@ sub singletest {
     if(loadtest("${TESTDIR}/test${testnum}")) {
         if($verbose) {
             # this is not a test
-            print "$testnum doesn't look like a test case!\n";
+            print "RUN: $testnum doesn't look like a test case!\n";
+        }
+        return -1;
+    }
+
+    my $serverproblem = serverfortest($testnum);
+
+    if($serverproblem) {
+        # there's a problem with the server, don't run
+        # this particular server, but count it as "skipped"
+        if($serverproblem> 1) {
+            print "RUN: test case $testnum couldn't run!\n";
+            $problems++;
+        }
+        else {
+            $skipped++;
         }
         return -1;
     }
@@ -460,7 +537,7 @@ sub singletest {
             }else {
                 print "$testnum requires $requires, which is not set; skipping\n";
                 $skipped++;
-                return 0;  # look successful
+                return -1;  # return test-not-run
             }
         }
     }
@@ -571,7 +648,8 @@ sub singletest {
         subVariables \$fileContent;
 #        print "DEBUG: writing file " . $filename . "\n";
         open OUTFILE, ">$filename";
-        print OUTFILE   $fileContent;
+        binmode OUTFILE; # for crapage systems, use binary       
+        print OUTFILE $fileContent;
         close OUTFILE;
     }
 
@@ -668,9 +746,8 @@ sub singletest {
         # verify redirected stdout
         my @actual = loadarray($STDOUT);
 
-        $res = compare(\@actual, \@validstdout);
+        $res = compare("stdout", \@actual, \@validstdout);
         if($res) {
-            print " stdout FAILED\n";
             return 1;
         }
         if(!$short) {
@@ -682,9 +759,8 @@ sub singletest {
     if(!$replyattr{'nocheck'} && @reply) {
         # verify the received data
         my @out = loadarray($CURLOUT);
-        $res = compare(\@out, \@reply);
+        $res = compare("data", \@out, \@reply);
         if ($res) {
-            print " data FAILED\n";
             return 1;
         }
         if(!$short) {
@@ -695,9 +771,8 @@ sub singletest {
     if(@upload) {
         # verify uploaded data
         my @out = loadarray("$LOGDIR/upload.$testnum");
-        $res = compare(\@out, \@upload);
+        $res = compare("upload", \@out, \@upload);
         if ($res) {
-            print " upload FAILED\n";
             return 1;
         }
         if(!$short) {
@@ -729,9 +804,8 @@ sub singletest {
             @protstrip= striparray( $_, \@protstrip);
         }
 
-        $res = compare(\@out, \@protstrip);
+        $res = compare("protocol", \@out, \@protstrip);
         if($res) {
-            print " protocol FAILED\n";
             return 1;
         }
         if(!$short) {
@@ -751,9 +825,8 @@ sub singletest {
         }
         my @generated=loadarray($filename);
 
-        $res = compare(\@generated, \@outfile);
+        $res = compare("output", \@generated, \@outfile);
         if($res) {
-            print " output FAILED\n";
             return 1;
         }
         if(!$short) {
@@ -772,6 +845,19 @@ sub singletest {
 
     unlink($FTPDCMD); # remove the instructions for this test
 
+    my @what = getpart("client", "killserver");
+    for(@what) {
+        my $serv = $_;
+        chomp $serv;
+        if($run{$serv}) {
+            stopserver($run{$serv}); # the pid file is in the hash table
+            $run{$serv}=""; # clear it
+        }
+        else {
+            print STDERR "RUN: The $serv server is not running\n";
+        }
+    }
+
     if($memory_debug) {
         if(! -f $memdump) {
             print "\n** ALERT! memory debuggin without any output file?\n";
@@ -806,8 +892,6 @@ sub singletest {
     return 0;
 }
 
-my %run;
-
 ##############################################################################
 # This function makes sure the right set of server is running for the
 # specified test case. This is a useful design when we run single tests as not
@@ -815,57 +899,38 @@ my %run;
 
 sub serverfortest {
     my ($testnum)=@_;
-    my @what;
 
-    if($testnum< 100) {
-        # 0 - 99 is for HTTP
-        push @what, "http";
-    }
-    elsif($testnum< 200) {
-        # 100 - 199 is for FTP
-        push @what, "ftp";
-    }
-    elsif($testnum< 300) {
-        # 200 - 299 is for FILE, no server!
-        push @what, "file";
-    }
-    elsif($testnum< 400) {
-        # 300 - 399 is for HTTPS
-        push @what, "https";
-    }
-    elsif($testnum< 500) {
-        # 400 - 499 is for FTPS
-        push @what, "ftps";
-    }
-
-    if(!@what) {
-        # load the test case file definition
-        if(loadtest("${TESTDIR}/test${testnum}")) {
-            if($verbose) {
-                # this is not a test
-                print "$testnum doesn't look like a test case!\n";
-            }
-            return 100;
-        }
-        @what = getpart("client", "server");
-
-        if(!$what[0]) {
-            warn "Test case $testnum has no server(s) specified!";
-            return 100;
+    # load the test case file definition
+    if(loadtest("${TESTDIR}/test${testnum}")) {
+        if($verbose) {
+            # this is not a test
+            print "$testnum doesn't look like a test case!\n";
         }
+        return 100;
     }
+    my @what = getpart("client", "server");
+
+    if(!$what[0]) {
+        warn "Test case $testnum has no server(s) specified!";
+        return 100;
+    }
+
     for(@what) {
         my $what = lc($_);
         $what =~ s/[^a-z]//g;
         if($what eq "ftp") {
             if(!$run{'ftp'}) {
-                runftpserver($verbose);
+                if(runftpserver($verbose)) {
+                    return 2; # error starting it
+                }
                 $run{'ftp'}=$FTPPIDFILE;
             }
         }
         elsif($what eq "http") {
             if(!$run{'http'}) {
-                runhttpserver($verbose);
+                if(runhttpserver($verbose)) {
+                    return 2; # error starting
+                }
                 $run{'http'}=$HTTPPIDFILE;
             }
         }
@@ -876,7 +941,9 @@ sub serverfortest {
                 return 1;
             }
             if(!$run{'ftp'}) {
-                runftpserver($verbose);
+                if(runftpserver($verbose)) {
+                    return 2; # error starting it
+                }
                 $run{'ftp'}=$FTPPIDFILE;
             }
             if(!$run{'ftps'}) {
@@ -894,7 +961,9 @@ sub serverfortest {
                 return 1;
             }
             if(!$run{'http'}) {
-                runhttpserver($verbose);
+                if(runhttpserver($verbose)) {
+                    return 2; # problems starting server
+                }
                 $run{'http'}=$HTTPPIDFILE;
             }
             if(!$run{'https'}) {
@@ -902,11 +971,12 @@ sub serverfortest {
                 $run{'https'}=$HTTPSPIDFILE;
             }
         }
+        elsif($what eq "none") {
+        }
         else {
             warn "we don't support a server for $what";
         }
     }
-    sleep 1; # give a second for the server(s) to startup
     return 0; # ok
 }
 
@@ -1042,20 +1112,14 @@ my $total=0;
 
 foreach $testnum (split(" ", $TESTCASES)) {
 
-    my $serverproblem = serverfortest($testnum);
-
-    if($serverproblem) {
-        # there's a problem with the server, don't run
-        # this particular server, but count it as "skipped"
-        $skipped++;
+    my $error = singletest($testnum);
+    if(-1 == $error) {
+        # not a test we can run
         next;
     }
 
-    my $error = singletest($testnum);
-    if(-1 != $error) {
-        # valid test case number
-        $total++;
-    }
+    $total++; # number of tests we've run
+
     if($error>0) {
         $failed.= "$testnum ";
         if(!$anyway) {
@@ -1065,7 +1129,7 @@ foreach $testnum (split(" ", $TESTCASES)) {
         }
     }
     elsif(!$error) {
-        $ok++;
+        $ok++; # successful test counter
     }
 
     # loop for next test
@@ -1085,18 +1149,21 @@ for(keys %run) {
 }
 
 if($total) {
-    printf("$ok tests out of $total reported OK: %d%%\n",
+    printf("TESTDONE: $ok tests out of $total reported OK: %d%%\n",
            $ok/$total*100);
 
     if($ok != $total) {
-        print "These test cases failed: $failed\n";
+        print "TESTFAIL: These test cases failed: $failed\n";
     }
 }
 else {
-    print "No tests were performed!\n";
+    print "TESTFAIL: No tests were performed!\n";
 }
 if($skipped) {
-    print "$skipped tests were skipped due to restraints\n";
+    print "TESTINFO: $skipped tests were skipped due to restraints\n";
+}
+if($problems) {
+    print "TESTINFO: $problems tests didn't run due to run-time problems\n";
 }
 if($total && ($ok != $total)) {
     exit 1;
